#!/usr/bin/env python3
"""pre_std
"""
import sys
import os
import subprocess
import argparse
#gero@bioinf.uni-leipzig.de

parser = argparse.ArgumentParser(description='DIEGO scanning routine for junction support files (TCGA formatted)')
parser.add_argument("-l", "--list", dest='my_list', required=True, type=argparse.FileType('r'),
                    help="sampleName tab-delimiter path/to/fileName")
parser.add_argument("-d", "--anno", dest='my_anno',required=True,
                    help="specifies the path to the annotation (bed 6 file of genebounderies with info "
                         "protein_coding;ENSG00000237683.5;AL627309.1 in column four.)")
parser.add_argument("-o", "--outdir", dest='my_outdir',
                    help="specifies the path to ouput directory (temp files are also created in this directory, "
                         "they are removed by rm *.rm) (default is '.')")

args = parser.parse_args()
anno = os.path.abspath(args.my_anno)
if(args.my_outdir):
    outPath = os.path.abspath(args.my_outdir)
else:
    outPath = os.path.abspath(".")
os.chdir(outPath) 

sampleH = {}


def build_datastructure():
    for line in args.my_list:
        columns = line.strip().split('\t')
        if columns[0] not in sampleH:
            sampleH[columns[0]] = columns[1]
        else:
            sys.stderr.write("error sample "+str(columns[0])+" is already present in data structure")
            sys.exit(1)


def intersect_with_anno():
    sampleId = sampleH.values()[0]
    subprocess.call('awk \'{if(NR>1){split($1,a,","); split(a[1],b,":"); '
                    'split(a[2],c,":"); print b[1]"\t"b[2]"\t"c[2]"\tname\t1\t"b[3]}}\' ' +
                    sampleId + ' > j_p.bed', shell=True)
    subprocess.call('intersectBed -a j_p.bed -b ' + anno +
                    ' -s -wa | sort | uniq -c | awk \'{if ($1>1){print $2"\t"$3"\t"$4"\t"$5"\t"$6"\t"$7}}\' > j_r1.bed',
                    shell=True)
    subprocess.call('intersectBed -a j_p.bed -b '+anno+' -v -wa > j_r2.bed', shell=True)
    subprocess.call('cat j_r1.bed j_r2.bed | sort -u > j_r.bed', shell=True)
    subprocess.call('awk -F $\'\t\' \'FNR==NR {a[$1":"$2":"$3]=1; next} '
                    '{if ($1":"$2":"$3 in a){hans+1}else{print $0}}\' j_r.bed j_p.bed > j.bed', shell=True)
    subprocess.call('intersectBed -a j.bed -b ' + anno +
                    ' -s -f 1.0 -wa -wb | awk \'{split($10,a,";"); split(a[2],b,"."); split(a[3],c,"."); '
                    'print $1":"$2"-"$3"\tN_w\t"b[1]"\t"c[1]}\' > junctions_gene.rm', shell=True)
    subprocess.call('rm j_p.bed j_r1.bed j_r2.bed j_r.bed j.bed', shell=True)


def build_table():
    sampleId = sampleH.values()[0]
    subprocess.call('echo junction > firstline.rm', shell=True)
    subprocess.call('awk \'{if (NR>1){split($1,a,",");split(a[1],b,":");split(a[2],c,":"); '
                    'print b[1]":"b[2]"-"c[2]}}\' '+sampleId+' > junctions.rm', shell=True)
    subprocess.call('echo type > type.rm', shell=True)
    for key, value in sampleH.iteritems():
        subprocess.call('echo '+key+' > '+key+'_firstline.rm', shell=True)
        subprocess.call('awk \'{if(NR>1){print $2}}\' '+value+' > '+key+'_tmp.rm', shell=True)
    subprocess.call('paste firstline.rm type.rm *_firstline.rm  | '
                    'awk \'{print $0 "\tgeneID\tgeneName"}\' > junction_table.txt', shell=True)
    subprocess.call('paste junctions.rm *_tmp.rm | sort -u > junction_table_body.rm',shell=True)
    subprocess.call('awk -F $\'\t\' \'FNR==NR {a[$1]=$0; next} {if ($1 in a){ split(a[$1],b,"\t"); '
                    'printf (b[1]"\t"b[2]); for(i=2; i<=NF; i++){printf "\t"$i}; '
                    'print "\t"b[3]"\t"b[4]}}\' junctions_gene.rm junction_table_body.rm >> junction_table.txt',
                    shell=True)
    subprocess.call('rm *.rm', shell=True)


build_datastructure()
intersect_with_anno()   
build_table()
sys.stderr.write('Now there should be a file named junction_table.txt in your specified out folder '
                 'that can be used as input for DIEGO.py \n')
